package gregapi.block.metatype;

import static gregapi.data.CS.*;
import static gregapi.data.OP.gearGtSmall;
import static gregapi.data.OP.rockGt;
import gregapi.block.IBlockToolable;
import gregapi.block.ToolCompat;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackSet;
import gregapi.data.*;
import gregapi.oredict.IOreDictListenerEvent;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictMaterial;
import gregapi.render.IIconContainer;
import gregapi.util.*;

import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.event.ForgeEventFactory;

public class BlockStones extends BlockMetaType implements IOreDictListenerEvent, IBlockToolable, Runnable {
	public static final boolean[]
	  MOSSY = {F,F,T,F,F,T,F,F,F,F,F,F,F,F,F,F}
	, SEALABLE = {F,F,F,T,F,T,T,T,T,T,T,T,T,T,T,T}
	, SPAWNABLE = {T,T,T,F,F,F,F,F,F,F,F,F,F,F,F,F}
	;
	
	public static final byte
	STONE =  0, SMOTH =  7,
	COBBL =  1, MCOBL =  2,
	BRICK =  3, CRACK =  4, MBRIK =  5, SBRIK = 12,
	CHISL =  6, WINDA = 13, WINDB = 14,
	RNFBR =  8, RSTBR =  9,
	TILES = 10, STILE = 11;
	
	public final OreDictMaterial mMaterial;
	public final ItemStackSet[] mEqualBlocks = {new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(), new ItemStackSet(),};
	
	public BlockStones(Class<? extends ItemBlock> aItemClass, Material aVanillaMaterial, SoundType aVanillaSoundType, String aName, String aDefaultLocalised, OreDictMaterial aMaterial, float aResistanceMultiplier, float aHardnessMultiplier, int aHarvestLevel, IIconContainer[] aIcons) {
		super(aItemClass, aVanillaMaterial == null ? Material.rock : aVanillaMaterial, aVanillaSoundType == null ? soundTypeStone : aVanillaSoundType, aName, aDefaultLocalised, aMaterial, aResistanceMultiplier, aHardnessMultiplier, aHarvestLevel, 15, aIcons);
		mMaterial = (aMaterial == null ? ANY.Stone : aMaterial);
		
		OP.crafting.addListener(this);
		GAPI_POST.mAfterInit.add(this);
		
		if (aDefaultLocalised != null) {
			LH.add(getUnlocalizedName()+".0.name", aDefaultLocalised);
			LH.add(getUnlocalizedName()+".1.name", aDefaultLocalised+" Cobblestone");
			LH.add(getUnlocalizedName()+".2.name", "Mossy "+aDefaultLocalised+" Cobblestone");
			LH.add(getUnlocalizedName()+".3.name", aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".4.name", "Cracked "+aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".5.name", "Mossy "+aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".6.name", "Chiseled "+aDefaultLocalised);
			LH.add(getUnlocalizedName()+".7.name", "Smooth "+aDefaultLocalised);
			LH.add(getUnlocalizedName()+".8.name", "Reinforced "+aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".9.name", "Redstoned "+aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".10.name", aDefaultLocalised+" Tiles");
			LH.add(getUnlocalizedName()+".11.name", "Small "+aDefaultLocalised+" Tiles");
			LH.add(getUnlocalizedName()+".12.name", "Small "+aDefaultLocalised+" Bricks");
			LH.add(getUnlocalizedName()+".13.name", aDefaultLocalised+" Windmill Tiles A");
			LH.add(getUnlocalizedName()+".14.name", aDefaultLocalised+" Windmill Tiles B");
		}
		
		OM.data(ST.make(this, 1, RNFBR), new OreDictItemData(mMaterial, U, ANY.Iron, OP.stick.mAmount));
		OM.data(ST.make(this, 1, RSTBR), new OreDictItemData(mMaterial, U, MT.Redstone, OP.dust.mAmount));
		
		if (mMaterial != ANY.Stone) {
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, STONE));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, COBBL));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, MCOBL));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, BRICK));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, CRACK));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, MBRIK));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, CHISL));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, SMOTH));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, TILES));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, STILE));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, SBRIK));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, WINDA));
			OM.reg_(OP.stone, mMaterial, ST.make(this, 1, WINDB));
		}
		
		for (int i = 0; i < mMaxMeta; i++) mEqualBlocks[i].add(ST.make(this, 1, i));
	}
	
	@Override
	protected BlockMetaType makeSlab(Class<? extends ItemBlock> aItemClass, Material aVanillaMaterial, SoundType aVanillaSoundType, String aName, String aDefaultLocalised, OreDictMaterial aMaterial, float aResistanceMultiplier, float aHardnessMultiplier, int aHarvestLevel, int aCount, IIconContainer[] aIcons, byte aSlabType, BlockMetaType aBlock) {
		return new BlockStones(aItemClass, aVanillaMaterial == null ? Material.rock : aVanillaMaterial, aVanillaSoundType == null ? soundTypeStone : aVanillaSoundType, aName, aDefaultLocalised, aMaterial, aResistanceMultiplier, aHardnessMultiplier, aHarvestLevel, aCount, aIcons, aSlabType, aBlock);
	}
	
	protected BlockStones(Class<? extends ItemBlock> aItemClass, Material aVanillaMaterial, SoundType aVanillaSoundType, String aName, String aDefaultLocalised, OreDictMaterial aMaterial, float aResistanceMultiplier, float aHardnessMultiplier, int aHarvestLevel, int aCount, IIconContainer[] aIcons, byte aSlabType, BlockMetaType aBlock) {
		super(aItemClass, aVanillaMaterial == null ? Material.rock : aVanillaMaterial, aVanillaSoundType == null ? soundTypeStone : aVanillaSoundType, aName, aDefaultLocalised, aMaterial, aResistanceMultiplier, aHardnessMultiplier, aHarvestLevel, aCount, aIcons, aSlabType, aBlock);
		mMaterial = (aMaterial == null ? ANY.Stone : aMaterial);
		
		if (aDefaultLocalised != null) {
			LH.add(getUnlocalizedName()+".0.name", aDefaultLocalised+" Slab");
			LH.add(getUnlocalizedName()+".1.name", aDefaultLocalised+" Cobblestone Slab");
			LH.add(getUnlocalizedName()+".2.name", "Mossy "+aDefaultLocalised+" Cobblestone Slab");
			LH.add(getUnlocalizedName()+".3.name", aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".4.name", "Cracked "+aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".5.name", "Mossy "+aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".6.name", "Chiseled "+aDefaultLocalised+" Slab");
			LH.add(getUnlocalizedName()+".7.name", "Smooth "+aDefaultLocalised+" Slab");
			LH.add(getUnlocalizedName()+".8.name", "Reinforced "+aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".9.name", "Redstoned "+aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".10.name", aDefaultLocalised+" Tiles Slab");
			LH.add(getUnlocalizedName()+".11.name", "Small "+aDefaultLocalised+" Tiles Slab");
			LH.add(getUnlocalizedName()+".12.name", "Small "+aDefaultLocalised+" Bricks Slab");
			LH.add(getUnlocalizedName()+".13.name", aDefaultLocalised+" Windmill Tiles A Slab");
			LH.add(getUnlocalizedName()+".14.name", aDefaultLocalised+" Windmill Tiles B Slab");
		}
		OM.data(ST.make(this, 1, STONE), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, COBBL), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, MCOBL), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, BRICK), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, CRACK), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, MBRIK), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, CHISL), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, SMOTH), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, RNFBR), new OreDictItemData(mMaterial, U2, ANY.Iron, OP.stick.mAmount / 2));
		OM.data(ST.make(this, 1, RSTBR), new OreDictItemData(mMaterial, U2, MT.Redstone, OP.dust.mAmount / 2));
		OM.data(ST.make(this, 1, TILES), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, STILE), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, SBRIK), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, WINDA), new OreDictItemData(mMaterial, U2));
		OM.data(ST.make(this, 1, WINDB), new OreDictItemData(mMaterial, U2));
//		OM.data(ST.make(this, 1,15), new OreDictItemData(mMaterial, U2));
		
		for (int i = 0; i < mMaxMeta; i++) mEqualBlocks[i].add(ST.make(this, 1, i));
	}
	
	@Override
	public void run() {
		RM.pack(rockGt.mat(mMaterial, 4), ST.make(this, 1, COBBL));
		
		CR.shaped(gearGtSmall.mat(mMaterial, 1), CR.DEF_NAC, "P ", " f", 'P', OP.stone.dat(mMaterial));
		CR.shaped(ST.make(this, 1, COBBL), CR.DEF_NAC, "XX", "XX", 'X', OP.rockGt.dat(mMaterial));
		
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[STONE]) {
		if (FL.Mana_TE.exists())
		RM.Bath			.addRecipe1(T,  0, 16					, tStack.toStack(), FL.Mana_TE.make(1), NF, ST.make(this, 1, CHISL));
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.Crusher		.addRecipe1(T, 16,  25+mHarvestLevel* 25, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, SMOTH));
		CR.shaped(ST.make(this, 4, BRICK), CR.DEF_NAC, "XX", "XX", 'X', tStack.toStack());
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[COBBL]) {
		if (FL.Mana_TE.exists())
		RM.Bath			.addRecipe1(T,  0, 16					, tStack.toStack(), FL.Mana_TE.make(1), NF, ST.make(this, 1, MCOBL));
		RM.Hammer		.addRecipe1(T, 16, 16,  8000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16,	25+mHarvestLevel* 25, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[MCOBL]) {
		RM.Hammer		.addRecipe1(T, 16, 16,  8000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16,	25+mHarvestLevel* 25, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[BRICK]) {
		if (FL.Mana_TE.exists())
		RM.Bath			.addRecipe1(T,  0, 16					, tStack.toStack(), FL.Mana_TE.make(1), NF, ST.make(this, 1, MBRIK));
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		CR.shaped(ST.make(this, 1, CRACK), CR.DEF_NAC, "h" , "X" , 'X', tStack.toStack());
		CR.shaped(ST.make(this, 1, RNFBR), CR.DEF_MIR, "Se", "X ", 'X', tStack.toStack(), 'S', OP.stick.dat(ANY.Iron));
		CR.shaped(ST.make(this, 1, RSTBR), CR.DEF_NAC, "Dh", "X ", 'X', tStack.toStack(), 'D', OP.dust.dat(MT.Redstone));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[CRACK]) {
		RM.Hammer		.addRecipe1(T, 16, 16,  7000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[MBRIK]) {
		RM.Hammer		.addRecipe1(T, 16, 16,  7000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[CHISL]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, COBBL));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		CR.shaped(ST.make(this, 1, CHISL), CR.DEF_NAC, "y" , "X" , 'X', tStack.toStack());
		CR.shaped(ST.make(this, 4, BRICK), CR.DEF_NAC, "XX", "XX", 'X', tStack.toStack());
		CR.shaped(ST.make(this, 2, TILES), CR.DEF_NAC, "X" , "X" , 'X', tStack.toStack());
		CR.shaped(ST.make(this, 2, STILE), CR.DEF_NAC, "XX"      , 'X', tStack.toStack());
		CR.shaped(ST.make(this, 2, SBRIK), CR.DEF_NAC, "X ", " X", 'X', tStack.toStack());
		CR.shaped(ST.make(this, 2, WINDA), CR.DEF_NAC, " X", "X ", 'X', tStack.toStack());
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[RNFBR]) {
		RM.Hammer		.addRecipe1(T, 16, 16,  7000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16, 200+mHarvestLevel*200, tStack.toStack(), ST.make(this, 1, COBBL), OM.dust(MT.Fe, OP.stick.mAmount));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[RSTBR]) {
		RM.Hammer		.addRecipe1(T, 16, 16,  7000			, tStack.toStack(), OP.rockGt.mat(mMaterial, 4));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK), OM.dust(MT.Redstone));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[TILES]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[STILE]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SBRIK]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[WINDA]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		CR.shapeless(ST.make(this, 1, WINDB), CR.DEF_NAC, new Object[] {tStack.toStack()});
		}
		for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[WINDB]) {
		RM.Hammer		.addRecipe1(T, 16, 16					, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.Crusher		.addRecipe1(T, 16,  50+mHarvestLevel* 50, tStack.toStack(), ST.make(this, 1, CRACK));
		RM.add_smelting(tStack.toStack(), ST.make(this, 1, STONE));
		CR.shapeless(ST.make(this, 1, WINDA), CR.DEF_NAC, new Object[] {tStack.toStack()});
		}
	}
	
	@Override
	public void onOreRegistration(OreDictRegistrationContainer aEvent) {
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_White])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 64, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, CHISL));
			}
		}
		
		
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Red])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, TILES));
			}
		}
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Green])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, STILE));
			}
		}
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Blue])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, WINDA));
			}
		}
		
		
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Cyan])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, BRICK));
			}
		}
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Magenta])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, SBRIK));
			}
		}
		if (aEvent.mOreDictName.equals(DYE_OREDICTS_LENS[DYE_INDEX_Yellow])) {
			for (ItemStackContainer tStack : (ItemStackSet<ItemStackContainer>)mEqualBlocks[SMOTH]) {
				RM.LaserEngraver.addRecipe2(T, 16, 16, tStack.toStack(), ST.amount(0, aEvent.mStack), ST.make(this, 1, WINDB));
			}
		}
	}
	
	@Override
	public int getItemStackLimit(ItemStack aStack) {
		switch (ST.meta(aStack)) {
		case  0: return UT.Code.bindStack(OP.stone				.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  1: return UT.Code.bindStack(OP.stoneCobble		.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  2: return UT.Code.bindStack(OP.stoneMossy			.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  4: return UT.Code.bindStack(OP.stoneCracked		.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  5: return UT.Code.bindStack(OP.stoneMossyBricks	.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  6: return UT.Code.bindStack(OP.stoneChiseled		.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		case  7: return UT.Code.bindStack(OP.stonePolished		.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		default: return UT.Code.bindStack(OP.stoneBricks		.mDefaultStackSize * (mBlock.mBlock == mBlock ? 1 : 2));
		}
	}
	
	@Override
	public long onToolClick(String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, World aWorld, byte aSide, int aX, int aY, int aZ, float aHitX, float aHitY, float aHitZ) {
		byte aMeta = (byte)aWorld.getBlockMetadata(aX, aY, aZ);
		if (aTool.equals(TOOL_prospector)) return aMeta == STONE && ToolCompat.prospectStone(this, aMeta, aQuality, aChatReturn, aWorld, aSide, aX, aY, aZ) ? 10000 : 0;
		if (aTool.equals(TOOL_chisel)) {
			switch(aMeta) {
			case BRICK: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, CRACK, 3); return mBlock == this ? 10000 : 5000; // Bricks to Cracked Brick
			case SMOTH: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, CHISL, 3); return mBlock == this ? 10000 : 5000; // Smooth to Chiseled
			case TILES: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, STILE, 3); return mBlock == this ? 10000 : 5000; // Tile to Small Tile
//			case STILE: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, CRKST, 3); return mBlock == this ? 10000 : 5000; // Small Tile to Broken Small Tile?
			case SBRIK: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, STILE, 3); return mBlock == this ? 10000 : 5000; // Small Bricks to Small Tile
			case WINDA: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, WINDB, 3); return mBlock == this ? 10000 : 5000;
			case WINDB: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, WINDA, 3); return mBlock == this ? 10000 : 5000;
			}
		}
		if (aTool.equals(TOOL_file)) {
			switch(aMeta) {
			case STONE: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, SMOTH, 3); return mBlock == this ? 10000 : 5000; // Stone to Smooth
			case BRICK: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, SBRIK, 3); return mBlock == this ? 10000 : 5000; // Bricks to Small Bricks
			case TILES: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, STILE, 3); return mBlock == this ? 10000 : 5000; // Tile to Small Tile
			case SBRIK: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, STILE, 3); return mBlock == this ? 10000 : 5000; // Small Bricks to Small Tile
			case WINDA: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, WINDB, 3); return mBlock == this ? 10000 : 5000;
			case WINDB: aWorld.setBlockMetadataWithNotify(aX, aY, aZ, WINDA, 3); return mBlock == this ? 10000 : 5000;
			}
		}
		if (aTool.equals(TOOL_drill)) {
			if (mBlock == this && aPlayer instanceof EntityPlayer && aMeta == BRICK) {
				for (int i = 0; i < ((EntityPlayer)aPlayer).inventory.mainInventory.length; i++) {
					int tIndex = ((EntityPlayer)aPlayer).inventory.mainInventory.length-i-1;
					ItemStack tStack = ((EntityPlayer)aPlayer).inventory.mainInventory[tIndex];
					if (OM.is("stickAnyIronOrSteel", tStack)) {
						if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, RNFBR, 3)) {
							if (!UT.Entities.hasInfiniteItems(aPlayer)) {
								if (--tStack.stackSize <= 0) {
									ForgeEventFactory.onPlayerDestroyItem(((EntityPlayer)aPlayer), tStack);
									((EntityPlayer)aPlayer).inventory.mainInventory[tIndex] = null;
								}
								if (((EntityPlayer)aPlayer).openContainer != null) ((EntityPlayer)aPlayer).openContainer.detectAndSendChanges();
							}
							return 10000;
						}
						break;
					}
				}
			}
		}
		return ToolCompat.onToolClick(this, aTool, aRemainingDurability, aQuality, aPlayer, aChatReturn, aPlayerInventory, aSneaking, aStack, aWorld, aSide, aX, aY, aZ, aHitX, aHitY, aHitZ);
	}
	
	@Override
	public float getBlockHardness(World aWorld, int aX, int aY, int aZ) {
		switch(aWorld.getBlockMetadata(aX, aY, aZ)) {
		case RNFBR: return Blocks.stone.getBlockHardness(aWorld, aX, aY, aZ) * mHardnessMultiplier * 2;
		default   : return Blocks.stone.getBlockHardness(aWorld, aX, aY, aZ) * mHardnessMultiplier;
		}
	}
	
	@Override
	public float getExplosionResistance(int aMeta) {
		switch(aMeta) {
		case RNFBR: return Blocks.stone.getExplosionResistance(null) * mResistanceMultiplier * 2;
		default   : return Blocks.stone.getExplosionResistance(null) * mResistanceMultiplier;
		}
	}
	
	@Override
	public void updateTick2(World aWorld, int aX, int aY, int aZ, Random aRandom) {
		if (!aWorld.isRemote && WD.burning(aWorld, aX, aY, aZ)) switch(aWorld.getBlockMetadata(aX, aY, aZ)) {
		case MCOBL: aWorld.setBlock(aX, aY, aZ, this, COBBL, 3); break;
		case MBRIK: aWorld.setBlock(aX, aY, aZ, this, BRICK, 3); break;
		}
	}
	
	@Override public boolean isSealable(int aMeta, byte aSide) {return SEALABLE[aMeta] && super.isSealable(aMeta, aSide);}
	@Override public int isProvidingWeakPower(IBlockAccess aWorld, int aX, int aY, int aZ, int aSide) {return aWorld.getBlockMetadata(aX, aY, aZ) == RSTBR ? 15 : 0;}
	@Override public boolean shouldCheckWeakPower(IBlockAccess aWorld, int aX, int aY, int aZ, int aSide) {return aWorld.getBlockMetadata(aX, aY, aZ) != RSTBR;}
	@Override public void onNeighborBlockChange2(World aWorld, int aX, int aY, int aZ, Block aBlock) {if (MOSSY[UT.Code.bind4(aWorld.getBlockMetadata(aX, aY, aZ))] && WD.burning(aWorld, aX, aY, aZ)) aWorld.scheduleBlockUpdate(aX, aY, aZ, this, tickRate(aWorld));}
	@Override public void onBlockAdded2(World aWorld, int aX, int aY, int aZ) {if (MOSSY[UT.Code.bind4(aWorld.getBlockMetadata(aX, aY, aZ))] && WD.burning(aWorld, aX, aY, aZ)) aWorld.scheduleBlockUpdate(aX, aY, aZ, this, tickRate(aWorld));}
	@Override public int tickRate(World aWorld) {return 100;}
	@Override public int damageDropped(int aMeta) {return aMeta==0?1:aMeta;}
	@Override public boolean canCreatureSpawn(int aMeta) {return mBlock == this && SPAWNABLE[aMeta];}
	@Override public int getFlammability(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return 0;}
	@Override public boolean isFlammable(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return MOSSY[UT.Code.bind4(aWorld.getBlockMetadata(aX, aY, aZ))];}
	@Override public int getFireSpreadSpeed(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return MOSSY[UT.Code.bind4(aWorld.getBlockMetadata(aX, aY, aZ))]?3000:0;}
	@Override public boolean isFireSource(World aWorld, int aX, int aY, int aZ, ForgeDirection aSide) {return MOSSY[UT.Code.bind4(aWorld.getBlockMetadata(aX, aY, aZ))];}
	@Override public boolean isReplaceableOreGen(World aWorld, int aX, int aY, int aZ, Block aTarget) {return (aTarget == this || (aY <= 6 && aTarget == Blocks.stone)) && aWorld.getBlockMetadata(aX, aY, aZ) == STONE;}
}