package gregapi.worldgen;

import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import gregapi.block.IBlockPlacable;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.HashSetNoNulls;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackMap;
import gregapi.data.CS.BlocksGT;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.UT;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;

public class StoneLayer {
	public final Block mBlock;
	public final byte mMetaData;
	public final OreDictMaterial mMaterial;
	public final ItemStackContainer mStack;
	public final List<StoneLayerOres> mOres;
	public IBlockPlacable mBlockOre, mBlockSmallOre;
	
	public StoneLayer(Block aBlock, OreDictMaterial aMaterial, StoneLayerOres... aOres) {
		this(aBlock, 0, aMaterial, aOres);
	}
	public StoneLayer(Block aBlock, long aMetaData, OreDictMaterial aMaterial, StoneLayerOres... aOres) {
		mBlock = aBlock;
		mMaterial = aMaterial;
		mMetaData = UT.Code.bind4(aMetaData);
		mStack = new ItemStackContainer(mBlock, 1, mMetaData);
		mOres = new ArrayListNoNulls<StoneLayerOres>(F, aOres);
		mBlockOre = BlocksGT.stoneToNormalOres.get(mStack);
		mBlockSmallOre = BlocksGT.stoneToSmallOres.get(mStack);
	}
	
	/** List of Stone and Ore Blocks, that can simply be replaced by the Stone Layers. */
	public static final Set<Block> REPLACEABLE_BLOCKS = new HashSetNoNulls(F, Blocks.stone, Blocks.coal_ore, Blocks.iron_ore, Blocks.gold_ore, Blocks.diamond_ore, Blocks.emerald_ore, Blocks.lapis_ore, Blocks.redstone_ore, Blocks.lit_redstone_ore, Blocks.monster_egg);
	/** List of generateable Stone Layers, via ItemStack of the Stone Block, so that MetaData is usable. */
	public static final List<StoneLayer> LAYERS = new ArrayList();
	/** Whenever two Rock Types hit each other in WorldGen an Ore from the returned List will spawn. The first ones mentioned inside the List can override the chances for others by spawning before, so insert the lowest chances first and then the high chances. */
	public static final ItemStackMap<ItemStackContainer, ItemStackMap<ItemStackContainer, List<StoneLayerOres>>> MAP = new ItemStackMap();
	
	public static boolean put(Block aTop, Block aBottom, StoneLayerOres... aOreChances) {
		return put(new ItemStackContainer(aTop, 1, 0), new ItemStackContainer(aBottom, 1, 0), aOreChances);
	}
	public static boolean put(Block aTop, long aMetaTop, Block aBottom, long aMetaBottom, StoneLayerOres... aOreChances) {
		return put(new ItemStackContainer(aTop, 1, aMetaTop), new ItemStackContainer(aBottom, 1, aMetaBottom), aOreChances);
	}
	public static boolean put(ItemStackContainer aTop, ItemStackContainer aBottom, StoneLayerOres... aOreChances) {
		if (aOreChances.length <= 0) return F;
		ItemStackMap<ItemStackContainer, List<StoneLayerOres>> tMap = MAP.get(aTop);
		if (tMap == null) MAP.put(aTop, tMap = new ItemStackMap<ItemStackContainer, List<StoneLayerOres>>());
		List<StoneLayerOres> tList = tMap.get(aBottom);
		if (tList == null) tMap.put(aBottom, tList = new ArrayList<StoneLayerOres>(aOreChances.length));
		for (StoneLayerOres tMat : aOreChances) if (tMat != null) tList.add(tMat);
		return T;
	}
	
	public static List<StoneLayerOres> get(StoneLayer aTop, StoneLayer aBottom) {
		return get(aTop.mStack, aBottom.mStack);
	}
	public static List<StoneLayerOres> get(ItemStackContainer aTop, ItemStackContainer aBottom) {
		ItemStackMap<ItemStackContainer, List<StoneLayerOres>> tMap = MAP.get(aTop);
		if (tMap == null) return Collections.EMPTY_LIST;
		List<StoneLayerOres> tList = tMap.get(aBottom);
		if (tList == null) return Collections.EMPTY_LIST;
		return tList;
	}
}